;
; Rendering code for the eco 4K intro
;
; Author:
;	s_tec
;

%include "nasmgl.inc"
%include "macros.inc"
%include "const.inc"

		global	_demo_init
		global	_demo_init_tex
		global	_demo_init_eco
		global	_demo_init_danger
		global	_demo_init_swirl
		global	_demo_init_smoke
		global	_demo_draw
		global	_demo_update_eco
		global	_demo_update_danger
		global	_demo_update_swirl
		global	_demo_update_smoke
		global	_demo_update_chevron
		global	_demo_update_blades
		global	_demo_alpha
		global	_demo_gradient
		global	_demo_image
		global	_demo_particles

		extern	g_t
		extern	g_dt
		extern	g_temp
		extern	_synth_end
		extern	_main_exit

;-----------------------------------------------------------------------------
; DLL imports
;-----------------------------------------------------------------------------

		;opengl32.dll:
		import	glClear,		__imp__glClear@4
		import	glColor4f,		__imp__glColor4f@16
		import	glBindTexture,		__imp__glBindTexture@8
		import	glBlendFunc,		__imp__glBlendFunc@8
		import	glColorPointer,		__imp__glColorPointer@16
		import	glDisable,		__imp__glDisable@4
		import	glDisableClientState,	__imp__glDisableClientState@4
		import	glDrawArrays,		__imp__glDrawArrays@12
		import	glEnable,		__imp__glEnable@4
		import	glEnableClientState,	__imp__glEnableClientState@4
		import	glLoadIdentity,		__imp__glLoadIdentity@0
		import	glRotatef,		__imp__glRotatef@16
		import	glScalef,		__imp__glScalef@12
		import	glTexCoordPointer,	__imp__glTexCoordPointer@16
		import	glTranslatef,		__imp__glTranslatef@12
		import	glVertexPointer,	__imp__glVertexPointer@16

;-----------------------------------------------------------------------------
; Macros
;-----------------------------------------------------------------------------

%macro const 2
%ifdef pass_color
%1		equ	%2
%endif
%endm

%macro rect 5
%ifdef pass_color
		global	g_rect_%1
g_rect_%1:	dd	%3, %4
		dd	%2, %4
		dd	%2, %5
		dd	%3, %5
%endif
%endm

%macro color 3
%ifdef pass_rect
		global	g_color_%1
g_color_%1:	dd	%2, %2, %3, %3
%endif
%endm

%macro script 1
%ifdef pass_script
		global	g_script_%1
g_script_%1:
%endif
%endm

%macro alpha 2
%ifdef pass_script
		dd	_demo_alpha, %1, %2
%endif
%endm

%macro draw_grad 1
%ifdef pass_script
		dd	_demo_gradient, g_color_%1
%endif
%endm

%macro draw_image 2
%ifdef pass_script
		dd	_demo_image, g_rect_%1, k_tex_%2
%endif
%endm

%macro draw_part 2
%ifdef pass_script
		dd	_demo_particles, s_part_%1, k_part_%1_n, k_tex_%2
%endif
%endm

%macro update 1
%ifdef pass_script
		dd	_demo_update_%1
%endif
%endm

%macro ends 0
%ifdef pass_script
		dd	_synth_end
%endif
%endm

;-----------------------------------------------------------------------------
; Constants
;-----------------------------------------------------------------------------

		struc	part
.x:		resd	1
.y:		resd	1
.r:		resd	1
.angle:		resd	1
.alpha:		resd	1
.t:		resd	1
.p0:		resd	1
.p1:		resd	1
		endstruc

k_part_max_n	equ	1024			;Maximum number of quads
k_part_eco_n	equ	192
k_part_danger_n	equ	64
k_part_swirl_n	equ	64
k_part_smoke_n	equ	192
k_part_chev_n	equ	12
k_part_blades_n	equ	9

;-----------------------------------------------------------------------------
; Uninitialized data
;-----------------------------------------------------------------------------
		section	.bss

s_demo_tex:	resd	8*k_part_max_n
s_demo_verts:	resd	8*k_part_max_n
s_demo_colors:	resd	4*k_part_max_n

s_part_eco:	resb	part_size*k_part_eco_n
s_part_danger:	resb	part_size*k_part_danger_n
s_part_swirl:	resb	part_size*k_part_swirl_n
s_part_smoke:	resb	part_size*k_part_smoke_n
s_part_chev:	resb	part_size*k_part_chev_n

;-----------------------------------------------------------------------------
; Initialized data
;-----------------------------------------------------------------------------
		section	.data

%define pass_color
%include "demo.inc"
%undef pass_color

%define pass_rect
%include "demo.inc"
%undef pass_rect

%define pass_script
%include "demo.inc"
%undef pass_script

s_part_blades:	dd	1.5,     -0.5,   3.5,   0,       1.0, 0, 0, 0
		dd	1.5,     -0.5,   3.5,   2.09375, 1.0, 0, 0, 0
		dd	1.5,     -0.5,   3.5,   4.1875,  1.0, 0, 0, 0
		dd	-5.625,   1.375, 4.375, 0,       1.0, 0, 0, 0
		dd	-5.625,   1.375, 4.375, 2.09375, 1.0, 0, 0, 0
		dd	-5.625,   1.375, 4.375, 4.1875,  1.0, 0, 0, 0
		dd	-10.625, -2.375, 2.625, 0,       1.0, 0, 0, 0
		dd	-10.625, -2.375, 2.625, 2.09375, 1.0, 0, 0, 0
		dd	-10.625, -2.375, 2.625, 4.1875,  1.0, 0, 0, 0

;-----------------------------------------------------------------------------
; Initializes OpenGL rendering resources
;-----------------------------------------------------------------------------
		section	.text
_demo_init:
		;Alpha bending mode:
		push	GL_ONE_MINUS_SRC_ALPHA	;dfactor
		push	GL_ONE			;sfactor
		call	glBlendFunc
		push	GL_BLEND		;cap
		call	glEnable

		;Scaling:
		call	glLoadIdentity
		push	0.0			;z
		push	0.11328125		;y
		push	0.0625			;x
		call	glScalef

		;Vertex arrays:
		push	GL_VERTEX_ARRAY		;array
		call	glEnableClientState

;-----------------------------------------------------------------------------
; Initializes the texture coordinate array
;-----------------------------------------------------------------------------
_demo_init_tex:
		mov	ecx, k_part_max_n
		mov	edi, s_demo_tex
.repeat:	fld1				;x
		fstp	dword[edi]		;
		add	edi, 4
		fld1				;y
		fstp	dword[edi]		;
		add	edi, 4
		fldz				;x
		fstp	dword[edi]		;
		add	edi, 4
		fld1				;y
		fstp	dword[edi]		;
		add	edi, 4
		fldz				;x
		fstp	dword[edi]		;
		add	edi, 4
		fldz				;y
		fstp	dword[edi]		;
		add	edi, 4
		fld1				;x
		fstp	dword[edi]		;
		add	edi, 4
		fldz				;y
		fstp	dword[edi]		;
		add	edi, 4
		loop	.repeat

;-----------------------------------------------------------------------------
; Initializes the eco particle system.
;-----------------------------------------------------------------------------
_demo_init_eco:
		mov	ecx, k_part_eco_n
		mov	esi, s_part_eco
.repeat:	random				;r
		fabs				;|r|
		fadd	dword[g_k1]		;r+=1
		fstp	dword[esi+part.t]	;
		add	esi, part_size
		loop	.repeat

;-----------------------------------------------------------------------------
; Initializes the danger particle system.
;-----------------------------------------------------------------------------
_demo_init_danger:
		mov	ecx, k_part_danger_n
		mov	esi, s_part_danger
.repeat:	random				;r
		fabs				;|r|
		fadd	dword[g_k1]		;r+=1
		fstp	dword[esi+part.t]	;
		add	esi, part_size
		loop	.repeat

;-----------------------------------------------------------------------------
; Initializes the swirl particle system.
;-----------------------------------------------------------------------------
_demo_init_swirl:
		mov	ecx, k_part_swirl_n
		mov	esi, s_part_swirl
.repeat:	random				;r
		fabs				;|r|
		fadd	dword[g_k1]		;r+=1
		fstp	dword[esi+part.t]	;
		add	esi, part_size
		loop	.repeat

;-----------------------------------------------------------------------------
; Initializes the smoke particle system.
;-----------------------------------------------------------------------------
_demo_init_smoke:
		mov	ecx, k_part_smoke_n
		mov	esi, s_part_smoke
.repeat:	random				;r
		fabs				;|r|
		fadd	dword[g_k1]		;r+=1
		fst	dword[esi+part.t]	;r
		fmul	dword[g_k4]		;r*=4
		fstp	dword[esi+part.p1]
		random				;r
		fmul	dword[g_k4]		;r*=8
		fstp	dword[esi+part.angle]	;
		add	esi, part_size
		loop	.repeat

		ret

;-----------------------------------------------------------------------------
; Renders one frame
;-----------------------------------------------------------------------------
		section	.text
_demo_draw:
		;Clear the canvas:
		push	GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT
		call	glClear

		mov	eax, [g_t]
		mov	ebp, g_script_eco
		sub	eax, k_length_eco
		jl	.go
		mov	ebp, g_script_power
		sub	eax, k_length_power
		jl	.go
		mov	ebp, g_script_stuff
		sub	eax, k_length_stuff
		jl	.go
		mov	ebp, g_script_recycle
		sub	eax, k_length_recycle
		jl	.go
		mov	ebp, g_script_wind
		sub	eax, k_length_wind
		jl	.go
		jmp	_main_exit
.go:		call	[ebp]
		ret

;-----------------------------------------------------------------------------
; Updates the eco particle system
;-----------------------------------------------------------------------------
_demo_update_eco:
		push	edi
		mov	ecx, k_part_eco_n
		mov	esi, s_part_eco
		fild	dword[g_dt]		;dt
		fmul	dword[g_k0p00003]	;dt*=rate

		;Advance the time:
.repeat:	fld	dword[esi+part.t]	;t,dt
		fadd	st1			;t+=dt,dt
		fld1				;1,t,dt
		fcomip	st1			;t,dt
		jnc	.skip			;Jump if t <= 1

		;Particle reset:
		fsub	dword[g_k1]		;t-=1,dt
		random				;r,...
		fmul	dword[g_k16]		;16*r,...
		fstp	dword[esi+part.x]	;...
		random				;r,...
		fmul	dword[g_k16]		;16*r,...
		fstp	dword[esi+part.y]	;,...

		;Update time, alpha and size:
.skip:		fst	dword[esi+part.t]	;t,dt
		fld1				;1,t,dt
		fsub	st1			;1-t,t,dt
		fstp	dword[esi+part.alpha]	;t,dt
		fmul	dword[g_k8]		;t*=8,dt
		fst	dword[esi+part.angle]	;dt
		fstp	dword[esi+part.r]	;dt

		;Loop:
		add	esi, part_size
		loop	.repeat

		add	ebp, 4
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Updates the danger particle system
;-----------------------------------------------------------------------------
_demo_update_danger:
		push	edi
		mov	ecx, k_part_danger_n
		mov	esi, s_part_danger
		fild	dword[g_dt]		;dt
		fmul	dword[g_k0p0001]	;dt*=rate

		;Advance the time:
.repeat:	fld	dword[esi+part.t]	;t,dt
		fadd	st1			;t+=dt,dt
		fld1				;1,t,dt
		fcomip	st1			;t,dt
		jnc	.skip			;Jump if t <= 1

		;Particle reset:
		fsub	dword[g_k1]		;t-=1,dt
		random				;r,...
		fmul	dword[g_k24]		;r*=24,...
		fstp	dword[esi+part.x]	;...
		random				;r,...
		fmul	dword[g_k4]		;r*=4,...
		fstp	dword[esi+part.y]	;,...

		;Update time, alpha and size:
.skip:		fst	dword[esi+part.t]	;t,dt
		fld1				;1,t,dt
		fsub	st1			;1-t,t,dt
		fstp	dword[esi+part.alpha]	;t,dt
		fmul	st0			;t*=t,dt
		fmul	dword[g_k8]		;t*=8,dt
		fstp	dword[esi+part.r]	;dt

		;Loop:
		add	esi, part_size
		loop	.repeat

		add	ebp, 4
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Updates the swirl particle system
;-----------------------------------------------------------------------------
_demo_update_swirl:
		push	edi
		mov	ecx, k_part_swirl_n
		mov	esi, s_part_swirl
		fild	dword[g_dt]		;dt
		fmul	dword[g_k0p00003]	;dt*=rate

		;Advance the time:
.repeat:	fld	dword[esi+part.t]	;t,dt
		fadd	st1			;t+=dt,dt
		fld1				;1,t,dt
		fcomip	st1			;t,dt
		jnc	.skip			;Jump if t <= 1

		;Particle reset:
		fsub	dword[g_k1]		;t-=1,dt
		random				;r,...
		fmul	dword[g_k4]		;16*r,...
		fstp	dword[esi+part.x]	;...
		random				;r,...
		fmul	dword[g_k2]		;16*r,...
		fstp	dword[esi+part.y]	;,...

		;Update time, alpha and size:
.skip:		fst	dword[esi+part.t]	;t,dt
		fld1				;1,t,dt
		fsub	st1			;1-t,t,dt
		fstp	dword[esi+part.alpha]	;t,dt
		fmul	dword[g_k16]		;t*=8,dt
		fst	dword[esi+part.r]	;dt
		fstp	dword[esi+part.angle]	;dt

		;Loop:
		add	esi, part_size
		loop	.repeat

		add	ebp, 4
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Updates the smoke particle system
;-----------------------------------------------------------------------------
_demo_update_smoke:
		push	edi
		mov	ecx, k_part_smoke_n
		mov	esi, s_part_smoke
		fild	dword[g_dt]		;dt
		fmul	dword[g_k0p0001]	;dt*=rate

		;Advance the time:
.repeat:	fld	dword[esi+part.t]	;t,dt
		fadd	st1			;t+=dt,dt
		fld1				;1,t,dt
		fcomip	st1			;t,dt
		jnc	.skip			;Jump if t <= 1

		;Particle reset:
		fsub	dword[g_k1]		;t-=1,dt

		;Update time, alpha, location, and size:
.skip:		fst	dword[esi+part.t]	;t,dt
		fld1				;1,t,dt
		fsub	st1			;1-t,t,dt
		fstp	dword[esi+part.alpha]	;t,dt
		fld	st0			;t,t,dt
		fmul	dword[esi+part.p1]	;t*=p1,t,dt
		fstp	dword[esi+part.y]	;t,dt
		fld	st0			;t,t,dt
		fmul	dword[esi+part.p1]	;t*=p1,t,dt
		fmul	st1			;t*=t,t,dt
		fsub	dword[g_k8]		;t-=8,t,dt
		test	ecx, 1
		jz	.skip2
		fsub	dword[g_k2]		;t-=2,t,dt
.skip2:		fstp	dword[esi+part.x]	;t,dt
		fmul	dword[g_k2]		;2*t,dt
		fstp	dword[esi+part.r]

		;Loop:
		add	esi, part_size
		loop	.repeat

		add	ebp, 4
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Updates the chevrons
;-----------------------------------------------------------------------------
_demo_update_chevron:
		push	edi
		mov	ecx, k_part_chev_n
		mov	esi, s_part_chev
		fld	dword[g_k4]		;d
		fild	dword[g_t]		;dt,d
		fmul	dword[g_k0p001]		;dt*=rate,d
		fmul	st1			;x*=d,d
		fprem				;x%d,d
		fsub	dword[g_k32]		;x-=32,d

.repeat:	fadd	st1			;x+=d,d
		fst	dword[esi+part.x]	;x,d
		fld	dword[g_k0p25]		;0.25,x,d
		fstp	dword[esi+part.alpha]	;x,d
		fld	dword[g_k8]		;8,x,d
		fstp	dword[esi+part.r]	;x,d

		;Loop:
		add	esi, part_size
		loop	.repeat

		add	ebp, 4
		fstp	st0			;d
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Updates the windmill blades.
;-----------------------------------------------------------------------------
_demo_update_blades:
		push	edi
		mov	ecx, k_part_blades_n
		mov	esi, s_part_blades
		fild	dword[g_dt]		;dt
		fmul	dword[g_k0p0001]	;dt*=rate

.repeat:	fld	dword[esi+part.r]	;r,dt
		fadd	dword[g_k4]		;r+4,dt
		fmul	st1			;dt*(r+4),dt
		fadd	dword[esi+part.angle]	;a+dt,dt
		fstp	dword[esi+part.angle]	;dt

		;Loop:
		add	esi, part_size
		loop	.repeat

		add	ebp, 4
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Sets up an alpha based on time.
;-----------------------------------------------------------------------------
%define		a_start	dword[ebp+4]		;First time, in samples
%define		a_len	dword[ebp+8]		;Length, in samples

_demo_alpha:
		push	edi

		;Alpha:
		fild	dword[g_t]		;t
		fisub	a_start			;t-t0
		fidiv	a_len			;a=(t-t0)/len

		;Clamp to 0:
		fldz				;0,a
		fcomip	st0, st1		;a
		jc	.skip0			;Jump if 0 < a
		fldz				;0,a
		fstp	st1			;a=0

		;Clamp to 1:
.skip0:		fld1				;1,a
		fcomip	st0, st1		;a
		jnbe	.skip1			;Jump if 1 > a
		fld1				;1,a
		fstp	st1			;a=1

		;Store:
.skip1:		fmul	st0			;a*a
		fstp	dword[g_temp]		;

		;Apply:
		push	dword[g_temp]		;alpha
		push	dword[g_temp]		;blue
		push	dword[g_temp]		;green
		push	dword[g_temp]		;red
		call	glColor4f

		add	ebp, 12
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Renders an static background gradient into the framebuffer
;-----------------------------------------------------------------------------
%define		a_color	dword[ebp+4]		;Color buffer address

_demo_gradient:
		push	edi

		;Texturing off:
		push	GL_TEXTURE_2D		;cap
		call	glDisable
		push	GL_TEXTURE_COORD_ARRAY	;array
		call	glDisableClientState
		push	GL_COLOR_ARRAY		;array
		call	glEnableClientState

		;Draw background texture:
		push	g_rect_screen		;pointer
		push	0			;stride
		push	GL_FLOAT		;type
		push	2			;size
		call	glVertexPointer
		push	a_color			;pointer
		push	0			;stride
		push	GL_UNSIGNED_BYTE	;type
		push	4			;size
		call	glColorPointer
		push	4			;count
		push	0			;first
		push	GL_QUADS		;mode
		call	glDrawArrays

		add	ebp, 8
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Renders an image into the framebuffer using the current alpha
;-----------------------------------------------------------------------------
%define		a_rect	dword[ebp+4]		;Rectangle vertices
%define		a_tex	dword[ebp+8]		;Texture

_demo_image:
		push	edi

		;Activate texture:
		push	a_tex			;texture
		push	GL_TEXTURE_2D		;target
		call	glBindTexture

		;Texturing on:
		push	GL_TEXTURE_2D		;cap
		call	glEnable
		push	GL_TEXTURE_COORD_ARRAY	;array
		call	glEnableClientState
		push	GL_COLOR_ARRAY		;array
		call	glDisableClientState

		;Draw:
		push	a_rect			;pointer
		push	0			;stride
		push	GL_FLOAT		;type
		push	2			;size
		call	glVertexPointer
		push	s_demo_tex		;pointer
		push	0			;stride
		push	GL_FLOAT		;type
		push	2			;size
		call	glTexCoordPointer
		push	4			;count
		push	0			;first
		push	GL_QUADS		;mode
		call	glDrawArrays

		add	ebp, 12
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Renders a particle system
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4]		;Buffer address
%define		a_size	dword[ebp+8]		;Buffer count
%define		a_tex	dword[ebp+12]		;Texture

_demo_particles:
		push	edi

		;Vertex array:
		mov	ecx, a_size
		mov	esi, a_name
		mov	edi, s_demo_verts

		;Find sines and cosines:
.repeat0:	fld	dword[esi+part.angle]	;a
		fsincos				;c,s
		fmul	dword[esi+part.r]	;c*=r,s
		fxch	st1			;s,c
		fmul	dword[esi+part.r]	;s*=r,c
		fld	st1			;c,s,c
		fmul	dword[g_k0p5]		;c2,s,c
		fld	st1			;s,c2,s,c
		fmul	dword[g_k0p5]		;s2,c2,s,c

		;x0 = x + s2 + c
		fld	dword[esi+part.x]	;x,s2,c2,s,c
		fadd	st1
		fadd	st4
		fstp	dword[edi]
		add	edi, 4

		;y0 = y - c2 + s
		fld	dword[esi+part.y]	;y,s2,c2,s,c
		fsub	st2
		fadd	st3
		fstp	dword[edi]
		add	edi, 4

		;x1 = x + s2
		fld	dword[esi+part.x]	;x,s2,c2,s,c
		fadd	st1
		fstp	dword[edi]
		add	edi, 4

		;y1 = y - c2
		fld	dword[esi+part.y]	;y,s2,c2,s,c
		fsub	st2
		fstp	dword[edi]
		add	edi, 4

		;x2 = x - s2
		fld	dword[esi+part.x]	;x,s2,c2,s,c
		fsub	st1
		fstp	dword[edi]
		add	edi, 4

		;y2 = y + c2
		fld	dword[esi+part.y]	;y,s2,c2,s,c
		fadd	st2
		fstp	dword[edi]
		add	edi, 4

		;x3 = x - s2 + c
		fld	dword[esi+part.x]	;x,s2,c2,s,c
		fsub	st1
		fadd	st4
		fstp	dword[edi]
		add	edi, 4

		;y3 = y + c2 + s
		fld	dword[esi+part.y]	;y,s2,c2,s,c
		fadd	st2
		fadd	st3
		fstp	dword[edi]
		add	edi, 4

		;Loop:
		fstp	st0
		fstp	st0
		fstp	st0
		fstp	st0
		add	esi, part_size
		loop	.repeat0

		;Color array:
		mov	ecx, a_size
		mov	esi, a_name
		mov	edi, s_demo_colors

		;Find alpha:
.repeat1:	fld	dword[esi+part.alpha]	;a
		fmul	dword[g_k255]		;a*=256
		fistp	dword[g_temp]		;
		mov	eax, [g_temp]
		mov	ah, al
		stosw
		stosw
		stosw
		stosw
		stosw
		stosw
		stosw
		stosw
		add	esi, part_size
		loop	.repeat1

		;Activate texture:
		push	a_tex		;texture
		push	GL_TEXTURE_2D		;target
		call	glBindTexture

		;Texturing on:
		push	GL_TEXTURE_2D		;cap
		call	glEnable
		push	GL_TEXTURE_COORD_ARRAY	;array
		call	glEnableClientState
		push	GL_COLOR_ARRAY		;array
		call	glEnableClientState

		;Draw particle system:
		push	s_demo_verts		;pointer
		push	0			;stride
		push	GL_FLOAT		;type
		push	2			;size
		call	glVertexPointer
		push	s_demo_tex		;pointer
		push	0			;stride
		push	GL_FLOAT		;type
		push	2			;size
		call	glTexCoordPointer
		push	s_demo_colors		;pointer
		push	0			;stride
		push	GL_UNSIGNED_BYTE	;type
		push	4			;size
		call	glColorPointer
		mov	eax, a_size
		shl	eax, 2
		push	eax			;count
		push	0			;first
		push	GL_QUADS		;mode
		call	glDrawArrays

		add	ebp, 16
		pop	edi
		jmp	[ebp]
