/*
 * Coded by
 *  William Swanson
 */

#include <Windows.h>
#include <gdiplus.h>
#include <math.h>
#include <d3d9.h>
using namespace Gdiplus;

void init();
void draw();

int const k_size_x    = 640;//1024;//1280;//1920;//
int const k_size_y    = 480;//768 ;//720 ;//1080;//
int const k_music_len = 120;

float const pi = 3.1415926535897932384f;

//Global variables:
IDirect3DDevice9 *g_device;
DWORD             g_time;

//Device options:
D3DPRESENT_PARAMETERS s_main_present = {
  0,                            //BackBufferWidth
  0,                            //BackBufferHeight
  D3DFMT_UNKNOWN,               //BackBufferFormat
  0,                            //BackBufferCount
  D3DMULTISAMPLE_NONE,          //MultiSampleType
  0,                            //MultiSampleQuality
  D3DSWAPEFFECT_DISCARD,        //SwapEffect
  0,                            //hDeviceWindow
  1,                            //Windowed
  0,                            //EnableAutoDepthStencil
  D3DFMT_UNKNOWN,               //AutoDepthStencilFormat
  0,                            //Flags
  0,                            //FullScreen_RefreshRateInHz
  D3DPRESENT_INTERVAL_DEFAULT   //PresentationInterval
};

GdiplusStartupInput s_main_gdip;

/**
 * Program entry point.
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  HWND hwnd;
  IDirect3D9 *d3d;
  ULONG gdip;
  DWORD start;
  MSG msg;

  //Create the window:
  hwnd = CreateWindowEx(
    0, "static", 0,
    WS_POPUP,
    (GetSystemMetrics(SM_CXSCREEN)-k_size_x)/2,
    (GetSystemMetrics(SM_CYSCREEN)-k_size_y)/2,
    k_size_x, k_size_y, 0, 0, 0, 0
  );

  //Initialize DirectX:
  s_main_present.hDeviceWindow = hwnd;
  d3d = Direct3DCreate9(D3D_SDK_VERSION);
  d3d->CreateDevice(
    D3DADAPTER_DEFAULT,
    D3DDEVTYPE_HAL,
    hwnd,
    D3DCREATE_SOFTWARE_VERTEXPROCESSING,
    &s_main_present,
    &g_device
  );
  ShowWindow(hwnd, SW_NORMAL);

  //Initialize GDI+:
  GdiplusStartup(&gdip, &s_main_gdip, 0);

  init();
  start = GetTickCount();

  //Wait for the user to press a key:
  do {
    g_time = GetTickCount() - start;
    draw();
    if(PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
      DispatchMessage(&msg);
  } while(msg.message != WM_KEYDOWN);

  //Clean up and exit:
  GdiplusShutdown(gdip);
  g_device->Release();
  d3d->Release();
  DestroyWindow(hwnd);
  return 0;
}

/**
 * Random number generator
 */
float random()
{
  static unsigned seed = 0x3e800000;
  seed = seed*1664525u + 1013904223u;
  return ((signed)seed)*(1.0f/0x7fffffff);
}

/**
 * Prepares graphics resources
 */
void init()
{
  Bitmap surface(256, 256, PixelFormat32bppARGB);
  Graphics g(&surface);

  g.Clear(Color::AliceBlue);

  GraphicsPath path;
  path.AddLine(2, 2, 100, 2);
  path.StartFigure();
  path.AddBezier(2, 50, 2, 100, 100, 100, 100, 50);
  PathData data;
  path.GetPathData(&data);

  Color c = Color::AliceBlue;
  SolidBrush brush(Color::Black);
  Pen p(Color::Black);
  g.DrawPath(&p, &path);

  GUID guid = {0x557CF406, 0x1A04, 0x11D3, {0x9A, 0x73, 0x00, 0x00, 0xF8, 0x1E, 0xF3, 0x2E}};
  surface.Save(L"test.png", &guid);
}

/**
 * Draws one frame
 */
void draw()
{
  //Begin drawing:
  g_device->BeginScene();

  //Clear the back buffer:
  g_device->Clear(0, 0, D3DCLEAR_TARGET, 0xfffff8f0, 0x3f800000, 0);

  //End drawing:
  g_device->EndScene();
  g_device->Present(0, 0, 0, 0);
}
