;
; 4K music implementation
; See music.txt for documentation.
;
; Author:
;	s_tec
;	necro2607
;

;Buffers:
buffer(music, k_music_len)
buffer(beginning_buf, 32)
;buffer(high_altchords_buf, 16)
buffer(beginning_chords_buf, 32)
buffer(beginning_percussion_buf, 32)
buffer(percussion_buf,16)
;buffer(percussion2_buf,16)
buffer(fatkicks_buf,4)
buffer(breakdrums1_buf,16)
buffer(breakdrums2_buf,16)
buffer(noise_buf,8)
;buffer(beginning_crushline1_2_buf,8)
;buffer(beginning_crushline3_buf,4)
;buffer(beginning_crushline4_buf,4)
buffer(crushline1_buf, 16)
buffer(crushline2_buf, 16)
buffer(oddchords_buf,16)
;buffer(oddbass1_buf,4)
;buffer(oddbass2_buf,4)
buffer(oddhighs_buf,16)
buffer(recycle_buf,16)
buffer(recycle_mainline_buf,16)
buffer(recycle_lead_buf,16)
buffer(raindrops_buf,16)
;buffer(endphrase_buf,8)
;buffer(endchords_buf,16)

const(k0,       0.0)
const(k0p00004, 0.0000400543212890625)
const(k0p002,   0.002)
const(k0p025,   0.025)
const(k0p075,   0.075)
const(k0p125,   0.125)
const(k0p1,     0.099609375)
const(k0p2,     0.19921875)
const(k0p25,    0.25)
const(k0p4,     0.3984375)
const(k0p5,     0.5)
const(k0p75,    0.75)
const(k0p99,    0.99)
const(k0p995,   0.995)
const(k0p999,   0.999)
const(k0p9999,  0.9999)
const(k8,       8.0)
const(k12,      12.0)
const(k64,      64.0)
const(k160,     160.0)
const(k2000,    2000.0)

;Lengths:
const(whole_note,        117760) ;Compresses better than 117600
const(half_note,         whole_note/2)
const(quarter_note,      whole_note/4)
const(eighth_note,       whole_note/8)
const(sixteenth_note,    whole_note/16)
const(thirtysecond_note, whole_note/32)
const(sixtyfourth_note,  whole_note/64)

;Odd lengths:
const(double_note,       whole_note*2)
const(quad_note,         whole_note*4)
const(three_eighth_note, quarter_note+eighth_note)
const(five_quarter_note, whole_note+quarter_note)
const(three_half_note,   whole_note+half_note)
const(nine_eighth_note,  whole_note+eighth_note)
const(back_eighth_note,  -eighth_note)
const(back5000,          -5000)
const(n1000,    1000)
const(n2000,    2000)
const(n4360,    4360) ;sixteenth_note-3000
const(n1360,    1360) ;sixteenth_note-6000
const(echo0,    7360)
const(echo1,    15872)

%define organ_vf b8

; Synth Patches:
%define crushbass(a, b) organ(0.25, a, sixteenth_note-3000, b)
%define tripz(a, b) organ(k0p1, a, sixteenth_note-6000, b)
%define recz(a, b) organ(0.0, organ_vf, a, b)

; Drum Patches:
%define KICK1 glitch(0.999, c5)
%define KICK2 glitch(0.995, c6)
%define KICK3 glitch(0.999, c4)

%define HIHAT1 glitch(0.99, a9)
%define HIHAT2 glitch(0.995, b9)
%define HIHAT3 organ(k0p4, a4f, thirtysecond_note/2, 0.002)
%define HIHAT4 organ(0.75, b2, thirtysecond_note/2, 0.002)
%define HIHAT5 organ(0.75, g1f, thirtysecond_note/2, g9)


%define SNARE1 organ(0.5, d3s, sixteenth_note, a9f)
%define SNARE2 organ(0.75, g1f, thirtysecond_note/2, g7)
%define SNARE3 hihat(0.999)

%define NOISE1 hihat(0.9999)
%define NOISE2 organ(0.75, c0, whole_note*2, c4)
%define NOISE3 organ(0.75, g1f, eighth_note, g9)
%define NOISE4 organ(0.75, d0, eighth_note, c2)

;Crush bass patterns, used for evil squelchy part after intro
script(cb1)

	crushbass(c2, c3)
	step(sixteenth_note)
	
ends

script(cb2)

	crushbass(c2s, c3s)
	step(sixteenth_note)
	
ends

script(cb3)

	crushbass(g1, g2)
	step(sixteenth_note)
	
ends

script(cb4)

	crushbass(g1s, g2s)
	step(sixteenth_note)
	
ends

;High bleepy "mysterious" synthlines
script(oddhighs1)
	tripz(c6, c5)
		step(sixteenth_note)
	tripz(e6f, e5f)
		step(sixteenth_note)
	tripz(g6, g5)
		step(sixteenth_note)
	tripz(c7, c6)
		step(sixteenth_note)
ends

script(oddhighs2)
	tripz(c6, c5)
		step(sixteenth_note)
	tripz(e6, e5)
		step(sixteenth_note)
	tripz(a6, a5)
		step(sixteenth_note)
	tripz(c7, c6)
		step(sixteenth_note)
ends

; this script is the drums for beginning percussion

script(beginning_percussion)
	KICK1
		step(sixteenth_note)
	HIHAT3
		step(sixteenth_note)
	HIHAT2
		step(eighth_note)
	SNARE2
		step(sixteenth_note)
	HIHAT3
		step(sixteenth_note)
	HIHAT2
		step(sixteenth_note)
	KICK2
		step(sixteenth_note)
	KICK1
		step(eighth_note)
	HIHAT2
		step(eighth_note)
	SNARE2
		step(sixteenth_note)
	HIHAT2
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
ends



script(recycle1)

	siner(eighth_note, g4)
		step(eighth_note)
	siner(eighth_note, b4f)
		step(eighth_note)
	siner(eighth_note, f5)
		step(eighth_note)
	siner(eighth_note, g4)
		step(eighth_note)
	siner(eighth_note, b4f)
		step(eighth_note)
	siner(quarter_note, d5)
		step(quarter_note+eighth_note)

ends

script(recycle2)

	siner(eighth_note, c5)
		step(eighth_note)
	siner(eighth_note, e5f)
		step(eighth_note)
	siner(eighth_note, g5)
		step(eighth_note)
	siner(eighth_note, c5)
		step(eighth_note)
	siner(eighth_note, e5f)
		step(eighth_note)
	siner(quarter_note, a5)
		step(quarter_note+eighth_note)	
	siner(eighth_note, c5)
		step(eighth_note)
	siner(eighth_note, e5f)
		step(eighth_note)
	siner(eighth_note, b5f)
		step(eighth_note)
	siner(eighth_note, c5)
		step(eighth_note)
	siner(eighth_note, e5f)
		step(eighth_note)
	siner(quarter_note, c6)
		step(quarter_note+eighth_note)

ends

script(recycle_mainline)
	crushbass(g2, g3)
		step(sixteenth_note)
	crushbass(g2, g3)
		step(sixteenth_note)
	crushbass(g2, g3)
		step(sixteenth_note)
	crushbass(b2f, b3f)
		step(sixteenth_note)
	crushbass(g2, g3)
		step(sixteenth_note)
	crushbass(g2, g3)
		step(sixteenth_note)
	crushbass(g2, g3)
		step(sixteenth_note)
	crushbass(g2, g3)
		step(sixteenth_note)
ends

script(recycle_mainline2)
	crushbass(c3, c4)
		step(sixteenth_note)
	crushbass(c3, c4)
		step(sixteenth_note)
	crushbass(c3, c4)
		step(sixteenth_note)
	crushbass(e3f, e4f)
		step(sixteenth_note)
	crushbass(c3, c4)
		step(sixteenth_note)
	crushbass(c3, c4)
		step(sixteenth_note)
	crushbass(c3, c4)
		step(sixteenth_note)
	crushbass(c3, c4)
		step(sixteenth_note)
ends


script(recycle_lead)
	recz(eighth_note, g4)
		step(eighth_note)
	recz(sixteenth_note, g4)
		step(sixteenth_note)
	recz(sixteenth_note, f4)
		step(sixteenth_note)
	recz(sixteenth_note, g4)
		step(quarter_note)
		
	recz(eighth_note, g4)
		step(eighth_note)
	recz(sixteenth_note, g4)
		step(sixteenth_note)
	recz(eighth_note, b4f)
		step(eighth_note)
	recz(sixteenth_note, g4)
		step(quarter_note)

ends

script(recycle_lead2)
	recz(eighth_note, c5)
		step(eighth_note)
	recz(sixteenth_note, c5)
		step(sixteenth_note)
	recz(sixteenth_note, b4f)
		step(sixteenth_note)
	recz(sixteenth_note, c5)
		step(quarter_note)
		
	recz(eighth_note, c5)
		step(eighth_note)
	recz(sixteenth_note, c5)
		step(sixteenth_note)
	recz(eighth_note, e5f)
		step(eighth_note)
	recz(sixteenth_note, c5)
		step(quarter_note)

ends
		

%if 0
script(raindrops_old)

	tink(0.999, g6)
		step(half_note+quarter_note)
	tink(0.999, a6)
		step(quarter_note)
	tink(0.999, b6f)
		step(whole_note)
	tink(0.999, c7)
		step(half_note+quarter_note)
	tink(0.999, b6f)
		step(quarter_note)
	tink(0.999, c7)
		step(half_note)
	tink(0.999, d7)
		step(half_note)

ends
%endif

script(raindrops)
		step(quarter_note*3)
	tink(0.9999, g7)
		step(whole_note)
	tink(0.9999, a7)
		step(whole_note)
	tink(0.9999, b7f)
		step(whole_note)
	tink(0.9999, c8)
ends
%if 0
script(endphrase)

	recz(eighth_note, g4)
		step(eighth_note)
	recz(eighth_note, b4f)
		step(eighth_note)
	recz(eighth_note, d5)
		step(eighth_note)
	recz(eighth_note, g5)
		step(eighth_note)
	recz(quarter_note, b5f)
		step(quarter_note)
	recz(quarter_note, c6)
		step(quarter_note)
	recz(half_note, d6)
		step(half_note)	
ends
%endif

script(endphrase)

	tink(0.9999, g6)
		step(eighth_note)
	;tink(0.9999, b6f)
		step(eighth_note)
	tink(0.9999, d7)
		step(eighth_note)
	;tink(0.9999, g7)
		step(eighth_note)
	tink(0.9999, g7)
		step(quarter_note)
	tink(0.9999, a7)
		step(quarter_note)
	tink(0.9999, b7)
		step(half_note)	
ends

%if 0
script(endchords_version1)
	siner(half_note, g4)
	siner(half_note, b4f)
	recz(half_note, d5)
		step(half_note)
	siner(half_note, d5)
	recz(half_note, g5)
		step(half_note)
	siner(whole_note, c5)
	siner(whole_note, e5f)
	recz(half_note, g5)
		step(half_note)
	recz(half_note, g5f)
		step(half_note)
	
	siner(whole_note*4, g3)
	;recz(whole_note, b3)
	siner(whole_note*3, d4)
	;recz(whole_note, g4)
	recz(whole_note*2, b4)
	;recz(whole_note*2, d5)
ends

	
script(endchords)
	siner(half_note, g3)
	siner(half_note, b3f)
	recz(half_note, d4)
		step(half_note)
	siner(half_note, d4)
	recz(half_note, g4)
		step(half_note)
	siner(half_note, d4)
	siner(whole_note, g4f)
	recz(whole_note, a4)
		step(half_note)
	recz(half_note, c4)
		step(half_note)
	
	siner(whole_note*4, g3)
	;recz(whole_note, b3)
	siner(whole_note*3, d4)
	;recz(whole_note, g4)
	recz(whole_note*2, b4)
	;recz(whole_note*2, d5)
ends
%endif
script(endchords_version3)
	siner(whole_note*4, g3)
	;recz(whole_note, b3)
	siner(whole_note*3, d4)
	;recz(whole_note, g4)
	recz(whole_note*2, b4)
	;recz(whole_note*2, d5)
ends
	
script(music)
	
	jump(beginning_buf)
	
		
	siner(whole_note+quarter_note, c2)
		step(whole_note)
	siner(whole_note+half_note, c2s)
		step(whole_note)
	siner(whole_note+half_note, g2)
		step(whole_note)
	organ(0.075, g2s, whole_note, g1s)
		step(whole_note)
		
	organ(k0p1, c2s, whole_note, c2)
		step(whole_note)
	organ(k0p2, c2, whole_note, c2s)
		step(whole_note)
	organ(k0p2, g2, whole_note, g1s)
		step(whole_note)	
	organ(k0p2, g2, whole_note+quarter_note, g2)
		step(half_note)	
	siner(half_note, g2)
		step(half_note)	
	organ(k0p4, g2, half_note, g1)
	;siner(whole_note+whole_note, g1)
		
%if 0		
	jump(high_altchords_buf)
	
	organ(0.002, organ_vf, whole_note, c5)
	organ(0.002, organ_vf, whole_note, d5s)
		step(whole_note)
	organ(0.002, organ_vf, whole_note, c5s)
	organ(0.002, organ_vf, whole_note, g5)
		step(whole_note)
	organ(0.002, organ_vf, whole_note, g5)
	organ(0.002, organ_vf, whole_note, d5s)
		step(whole_note)
	organ(0.002, organ_vf, whole_note, g5s)
	organ(0.002, organ_vf, whole_note, d5)
%endif

	
	jump(beginning_chords_buf)
		step(2000)
	organ(k0p1, c2, whole_note, c3)
		step(1000)
	organ(k0p1, g3, whole_note+eighth_note, g4)
		step(whole_note)
	organ(k0p1, c2s, whole_note+eighth_note, c3s)	
		step(1000)
	organ(k0p1, f3, whole_note+eighth_note, f4)
		step(-5000)
		step(whole_note)
	organ(k0p1, g2s, whole_note+eighth_note, g3s)	
		step(2000)
	organ(k0p1, c3, whole_note+eighth_note, c4)
		step(whole_note)
	organ(k0p1, g2, whole_note+eighth_note, g3)	
		step(2000)
	organ(k0p1, b2, whole_note+eighth_note, b3)
	
	



	jump(crushline1_buf)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
	
	jump(crushline2_buf)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		run(cb1)
		
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		run(cb2)
		
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		run(cb4)
		
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
		run(cb3)
	
	
	;Mysterious chords, lower stylez
	jump(oddchords_buf)
	
	organ(k0p1, e4f, whole_note+whole_note, e3f)
	organ(k0p1, g4, whole_note+whole_note, g3)
	organ(k0p1, c5, whole_note+whole_note, c4)
		step(whole_note+whole_note)
	organ(k0p1, e4, whole_note+whole_note, e3)
	organ(k0p1, a4, whole_note+whole_note, a3)
	organ(k0p1, c5, whole_note+whole_note, c4)
	
	
	;The mysterious high arp notes
	jump(oddhighs_buf)

	run(oddhighs1)
	run(oddhighs1)
	run(oddhighs1)
	run(oddhighs1)
	run(oddhighs1)
	run(oddhighs1)
	run(oddhighs1)
	run(oddhighs1)
	
	run(oddhighs2)
	run(oddhighs2)
	run(oddhighs2)
	run(oddhighs2)
	run(oddhighs2)
	run(oddhighs2)
	run(oddhighs2)
	run(oddhighs2)
	
	
	
	

	
	jump(beginning_percussion_buf)
	
	run(beginning_percussion)
	run(beginning_percussion)
	run(beginning_percussion)
	run(beginning_percussion)
	run(beginning_percussion)
	run(beginning_percussion)
	run(beginning_percussion)
	run(beginning_percussion)
	
	
	jump(percussion_buf)
	
	KICK1
		step(quarter_note)
	KICK2
		step(eighth_note)
	HIHAT3
		step(eighth_note)
	KICK1
		step(eighth_note)
	HIHAT3
		step(eighth_note)
	KICK2
		step(quarter_note)
	KICK1
		step(quarter_note)
	KICK2
		step(eighth_note)
	HIHAT3
		step(eighth_note)
	KICK1
		step(quarter_note)
	KICK2
		step(quarter_note)
	
	
	
	jump(noise_buf)
	
		step(half_note)
	NOISE1
		step(quarter_note)
	NOISE1
		step(quarter_note)
	NOISE3
	
	
	
	jump(fatkicks_buf)
	
	KICK3
		step(quarter_note)
	KICK3
		step(quarter_note)
	KICK3
		step(quarter_note)
	KICK3
		
		
		
		
	jump(breakdrums1_buf)
	
	KICK2
	HIHAT1
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	SNARE3
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	HIHAT4
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	SNARE3
	HIHAT2
		step(thirtysecond_note)
	HIHAT1	
		step(thirtysecond_note)
	HIHAT2
		step(sixteenth_note)
	KICK2
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	SNARE3
		step(sixteenth_note)
	HIHAT2
		step(sixteenth_note)
	KICK2
	HIHAT2
		step(thirtysecond_note)
	HIHAT1
		step(thirtysecond_note)
	KICK1
	HIHAT2
		step(thirtysecond_note)
	HIHAT2
		step(thirtysecond_note)
	KICK2
	HIHAT1
		step(thirtysecond_note)
	HIHAT1
		step(thirtysecond_note)
	KICK2
	HIHAT2
		step(thirtysecond_note)
	HIHAT1
		step(thirtysecond_note)

		
		
	jump (breakdrums2_buf)

	KICK2
	HIHAT1
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	SNARE3
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	HIHAT4
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	SNARE3
	HIHAT2
		step(thirtysecond_note)
	HIHAT1	
		step(thirtysecond_note)
	HIHAT2
		step(sixteenth_note)
	KICK2
		step(sixteenth_note)
	SNARE3
		step(sixteenth_note)
	KICK2
		step(sixteenth_note)
	HIHAT1
		step(sixteenth_note)
	HIHAT2
		step(sixteenth_note)
	KICK1
	HIHAT2
		step(thirtysecond_note)
	HIHAT2
		step(thirtysecond_note)
	KICK2
	HIHAT1
		step(thirtysecond_note)
	HIHAT1
		step(thirtysecond_note)
	KICK2
	HIHAT2
		step(thirtysecond_note)
	HIHAT1
		step(thirtysecond_note)
	
	jump(recycle_buf)
		run(recycle1)
		run(recycle1)
		
	jump(recycle_buf)
		step(whole_note*2)
		run(recycle2)
		
	jump(recycle_mainline_buf)
		run(recycle_mainline)
		run(recycle_mainline)
		run(recycle_mainline)
		run(recycle_mainline)
		run(recycle_mainline2)
		run(recycle_mainline2)
		run(recycle_mainline2)
		run(recycle_mainline2)

	jump(recycle_lead_buf)
		run(recycle_lead)
		run(recycle_lead)
			step(-eighth_note)
		run(recycle_lead2)
		run(recycle_lead2)
		
		
	jump(raindrops_buf)
		run(raindrops)
		
;	jump(endphrase_buf)
;		run(endphrase)
		
;	jump(endchords_buf)
;		run(endchords_version3)
	;---------------------------------------------------------------------
	; Chorus stage
	;---------------------------------------------------------------------
	chorus(beginning_buf, 160.0, k0p00004, 2000.0, 0.25)
	bandpass(beginning_buf,    k0p2, 1.36505266675, 0.5625)
	
	chorus(beginning_chords_buf, 160.0, 0.00002, 2000.0, 0.5)
	
	
	chorus(recycle_buf, 160.0, k0p00004, 2000.0, 0.25)
	echo(recycle_buf, 14720, 0.25)
	bandpass(recycle_buf,     0.075, 1.24702283350, 0.9025)
	
	bandpass(noise_buf,    0.5, 0.779605721479, 0.25)
	chorus(noise_buf, 160.0, k0p00004, 2000.0, 0.5)
	
;	chorus(high_altchords_buf, 160.0, k0p00004, 2000.0, 0.25)
;	bandpass(high_altchords_buf,    0.0735797, 1.24702283350, 0.9025)
;	echo(high_altchords_buf, 15872, 0.25)
	
	bitcrush(breakdrums1_buf, 12.0)
	
	echo(percussion_buf, 7360, 0.25)
	
	bandpass(beginning_percussion_buf,k0p2, 1.40515451739, 0.64)
	bitcrush(beginning_percussion_buf, 12.0)
	
	chorus(crushline1_buf, 160.0, k0p00004, 2000.0, 0.75)
	bitcrush(crushline1_buf, 12.0)
	echo(crushline1_buf, 7360, 0.25)
	
	chorus(crushline2_buf, 160.0, k0p00004, 2000.0, 0.75)
	bitcrush(crushline2_buf, 12.0)
	echo(crushline2_buf, 7360, 0.25)
	
	chorus(oddchords_buf, 160.0, k0p00004, 2000.0, 0.25)
	
	bitcrush(oddhighs_buf, 64.0)
	bandpass(oddhighs_buf,    0.25, 1.16940858222, 0.5625)
	echo(oddhighs_buf, 14720, 0.25)
	
	bitcrush(recycle_mainline_buf,12.0)
	echo(recycle_mainline_buf, 14720, 0.25)

	echo(recycle_lead_buf, 14720, 0.25)
	;bitcrush(raindrops_buf, 64.0)
	;echo(raindrops_buf, 7360, k0p2)
	;chorus(raindrops_buf, 160.0, k0p00004, 2000.0, k0p1)
	
;	chorus(endchords_buf, 160.0, k0p00004, 2000.0, 0.25)
;	echo(endchords_buf, 14720, 0.25)
;	bandpass(endchords_buf,0.075, 1.24702283350, 0.9025)
	
;	bitcrush(oddbass1_buf, 64.0)
;	bitcrush(oddbass2_buf, 64.0)
;	bandpass(oddbass1_buf,    0.278292, 1.16940858222, 0.5625)
;	bandpass(oddbass2_buf,    0.278292, 1.16940858222, 0.5625)
;	echo(oddbass1_buf, 7360, 0.25)
;	echo(oddbass2_buf, 15872, 0.25)

	jump(music)
	
	;intro section
		mix(beginning_buf, 0.75)
			step(whole_note*4)
		mix(beginning_chords_buf, 0.140625)
			step(whole_note*4)
			step(half_note)
			step(quarter_note)
		mix(beginning_buf, 0.75)
	
	; evil industrial section	
		mix(crushline1_buf, 0.25)
		mix(beginning_percussion_buf, k0p2)
			step(whole_note*4)
			
		mix(crushline2_buf, 0.25)	
			step(whole_note*4)

			
	;fast-paced section starts here
		mix(oddchords_buf, k0p1)
		mix(oddhighs_buf, k0p2)	
		mix(percussion_buf, k0p2)
		mix(noise_buf, 0.025)
		
			;step(whole_note*4)
	
			
			step(whole_note*2)
		mix(noise_buf, 0.025)
			step(whole_note*2)
		
		
		mix(oddchords_buf, k0p1)
		mix(oddhighs_buf, k0p2)
		mix(breakdrums1_buf, k0p2)
		
			step(whole_note)
		mix(breakdrums2_buf, k0p2)
			step(whole_note)
		mix(breakdrums1_buf, k0p2)
			step(whole_note)
		mix(breakdrums2_buf, k0p2)
			step(whole_note)
			
	;	mix(oddchords_buf, k0p1)
	;	mix(oddhighs_buf, k0p2)
	;	mix(breakdrums1_buf, k0p2)
	;	mix(noise_buf, 0.025)
	;	mix(fatkicks_buf, 0.25)
	;		step(whole_note)
	;	mix(breakdrums2_buf, k0p2)
	;	mix(fatkicks_buf, 0.25)
	;		step(whole_note)
	;	mix(breakdrums1_buf, k0p2)
	;	mix(noise_buf, 0.025)
	;	mix(fatkicks_buf, 0.25)
	;		step(whole_note)
	;	mix(breakdrums2_buf, k0p2)
	;	mix(fatkicks_buf, 0.25)
	;		step(whole_note)
			
		;short breaktime between sections
		step(half_note)
			
	;recycle section starts here
		mix(percussion_buf, 0.25)
		mix(recycle_buf, 0.75)
		mix(recycle_mainline_buf,0.125)
			step(whole_note*2)
		mix(percussion_buf, 0.25)
			step(whole_note*2)
		mix(fatkicks_buf, 0.25)
		mix(recycle_buf, 0.75)
		mix(recycle_mainline_buf,0.125)
		mix(recycle_lead_buf, 0.25)
			step(whole_note)
		mix(fatkicks_buf, 0.25)
			step(whole_note)
		mix(fatkicks_buf, 0.25)
			step(whole_note)
		mix(fatkicks_buf, 0.25)
			step(whole_note)
			
		mix(recycle_buf, 0.75)
		mix(recycle_mainline_buf,0.125)
		mix(recycle_lead_buf, 0.25)
		mix(beginning_percussion_buf, k0p2)
		mix(fatkicks_buf, 0.25)
			step(whole_note)
		mix(fatkicks_buf, 0.25)
			step(whole_note)
		mix(fatkicks_buf, 0.25)
			step(whole_note)
		mix(fatkicks_buf, 0.25)
			step(whole_note)
	
	
	; water-drops section starts here
		mix(recycle_buf, 0.75)
		mix(raindrops_buf, k0p075)
			step(whole_note*4)
		mix(recycle_buf, 0.75)
		mix(raindrops_buf, k0p075)
	;		step(whole_note*4)
	;	
	;	mix(recycle_buf, 0.75)
	;	mix(recycle_lead_buf, k0p075)
	;		step(whole_note*4)
	;	mix(recycle_lead_buf, k0p075)
	;	mix(raindrops_buf, k0p075)
	; ending phrase
	
;		mix(endphrase_buf, k0p075)
;			step(whole_note+half_note)
;		mix(endchords_buf, k0p4)
		
				
ends
