;
; 4K synthesizer instruments and effects implementations
;
; Author:
;	s_tec
;

%include "win32n.inc"
%include "macros.inc"
%include "const.inc"

		global	_synth_play
		global	_synth_end
		global	_synth_run
		global	_synth_jump
		global	_synth_step

		global	_synth_tink
		global	_synth_glitch
		global	_synth_hihat
		global	_synth_siner
		global	_synth_organ

		global	_synth_mix
		global	_synth_echo
		global	_synth_bitcrush
		global	_synth_bandpass
		global	_synth_chorus

		extern	g_temp
		extern	g_script_music
		extern	g_buffer_music

;-----------------------------------------------------------------------------
; DLL imports
;-----------------------------------------------------------------------------

		;winmm.dll:
		import	waveOutOpen,		__imp__waveOutOpen@24
		import	waveOutPrepareHeader,	__imp__waveOutPrepareHeader@12
		import	waveOutWrite,		__imp__waveOutWrite@12

%ifdef debug
		;kernel32.dll:
		import	CreateFileA,		__imp__CreateFileA@28
		import	WriteFile,		__imp__WriteFile@20
		import	CloseHandle,		__imp__CloseHandle@4
%endif

;-----------------------------------------------------------------------------
; Constants
;-----------------------------------------------------------------------------

s_synth_delay_size	equ	4096

;-----------------------------------------------------------------------------
; Uninitialized data
;-----------------------------------------------------------------------------
		section	.bss

		;Circular buffer:
s_synth_delay:	resd	s_synth_delay_size

;-----------------------------------------------------------------------------
; Initialized data
;-----------------------------------------------------------------------------
		section	.data

s_wave_header:		;Describes the audio buffer:
%ifdef debug
.lpData:		dd	g_buffer_music + k_wave_bps*k_music_start
.dwBufferLength:	dd	k_wave_size - k_wave_bps*k_music_start
%else
.lpData:		dd	g_buffer_music
.dwBufferLength:	dd	k_wave_size
%endif
.dwBytesRecorded:	dd	0
.dwUser:		dd	0
.dwFlags:		dd	0
.dwLoops:		dd	0
.lpNext:		dd	0
.reserved:		dd	0
	
s_wave_format:		;Describes the audio format:
.wFormatTag:		dw	WAVE_FORMAT_PCM
.nChannels:		dw	k_wave_chan
.nSamplesPerSec:	dd	k_wave_sps
.nAvgBytesPerSec:	dd	k_wave_bps
.nBlockAlign:		dw	k_wave_align
.wBitsPerSample:	dw	k_wave_bits
.cbSize			dw	0

%ifdef debug
s_wav_file:	db	"music.wav", 0
		align	4, db 0

s_wav_header:	;Wave file header:
		db	"RIFF"
		dd	k_wave_size + k_wav_head_size
		db	"WAVE"
		db	"fmt "
		dd	16
		dw	WAVE_FORMAT_PCM
		dw	k_wave_chan
		dd	k_wave_sps
		dd	k_wave_bps
		dw	k_wave_align
		dw	k_wave_bits
		db	"data"
		dd	k_wave_size
k_wav_head_size	equ	$-s_wave_header
%endif

;-----------------------------------------------------------------------------
; Audio generation
;-----------------------------------------------------------------------------
		section	.text
_synth_play:
		;Generate the audio:
		mov	ebp, g_script_music
		call	[ebp]

		;Convert from floats to integers:
		mov	ecx, k_music_len*k_wave_sps*k_wave_chan
		mov	esi, g_buffer_music
		mov	edi, g_buffer_music
.repeat:	fld	dword[esi]		;y
		fld1				;1,y
		fpatan				;y
		fsin				;y
		fmul	dword[g_k32768]
		fistp	dword[edi]
		add	esi, 4
		add	edi, 2
		loop	.repeat

;-----------------------------------------------------------------------------
; Wave file output
;-----------------------------------------------------------------------------
%ifdef debug
		;Open music.wav:
		push	0			;hTemplateFile
		push	0			;dwFlagsAndAttributes
		push	CREATE_ALWAYS		;dwCreationDisposition
		push	0			;lpSecurityAttributes
		push	0			;dwShareMode
		push	GENERIC_WRITE		;dwDesiredAccess
		push	s_wav_file		;lpFileName
		call	CreateFileA
		mov	esi, eax

		;Write the header:
		push	0			;lpOverlapped
		push	g_temp			;lpNumberOfBytesWritten
		push	k_wav_head_size		;nNumberOfBytesToWrite
		push	s_wav_header		;lpBuffer
		push	esi			;hFile
		call	WriteFile

		;Write the audio:
		push	0			;lpOverlapped
		push	g_temp			;lpNumberOfBytesWritten
		push	k_wave_size		;nNumberOfBytesToWrite
		push	g_buffer_music		;lpBuffer
		push	esi			;hFile
		call	WriteFile

		;Close the file:
		push	esi			;hObject
		call	CloseHandle
%endif
;-----------------------------------------------------------------------------
; Audio playback
;-----------------------------------------------------------------------------

		;Open the audio device:
		push	CALLBACK_NULL		;fdwOpen
		push	0			;dwCallbackInstance
		push	0			;dwCallback
		push	s_wave_format		;pwfx
		push	WAVE_MAPPER		;uDeviceID
		push	g_temp			;phwo
		call	waveOutOpen

		;Prepare the buffer:
		push	WAVEHDR_size		;cbwh
		push	s_wave_header		;pwh
		push	dword[g_temp]		;hwo
		call	waveOutPrepareHeader

		;Play the buffer:
		push	WAVEHDR_size		;cbwh
		push	s_wave_header		;pwh
		push	dword[g_temp]		;hwo
		call	waveOutWrite

;-----------------------------------------------------------------------------
; Returns from a script block. Also the return statement for _synth_play
;-----------------------------------------------------------------------------
_synth_end:	ret

;-----------------------------------------------------------------------------
; Calls another script block
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4]		;Script address

_synth_run:
		push	ebp
		mov	ebp, a_name
		call	[ebp]
		pop	ebp
		add	ebp, 8
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Moves the write pointer to the beginning of an audio buffer
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4]		;Buffer address

_synth_jump:
		mov	edi, a_name
		add	ebp, 8
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Advances the write pointer
;-----------------------------------------------------------------------------
%define		a_d	dword[ebp+4]		;Distance, in bytes

_synth_step:
		add	edi, a_d
		add	ebp, 8
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Generates a "tink" sound
;-----------------------------------------------------------------------------
%define		a_d	dword[ebp+4]		;Decay factor
%define		a_f	dword[ebp+8]		;Frequency

_synth_tink:
		push	edi
		fldz				;t
		fld1				;a,t

		;Envelope:
.repeat:	fmul	a_d			;a*=d,t

		;Oscillator:
		fldpi				;pi,a,t
		fmul	a_f			;f*=pi,a,t
		faddp	st2			;a,t+=f
		fld	st1			;t,a,t
		fsin				;sin(t),a,t

		;Output:
		fmul	st1			;a*sin(t),a,t
		fadd	dword[edi]		;y,a,t
		fstp	dword[edi]		;a,t
		add	edi, 4

		;Loop:
		fld	dword[g_k0p001]		;0.001,a,t
		fcomip	st0, st1		;a,t
		jc	.repeat
		
		add	ebp, 12
		fstp	st0			;t
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Generates a "glitch" sound
;-----------------------------------------------------------------------------
%define		a_d	dword[ebp+4]		;Decay factor
%define		a_f	dword[ebp+8]		;Frequency

_synth_glitch:
		push	edi
		fldz				;t
		fld1				;a,t

		;Envelope:
.repeat:	fmul	a_d			;a*=d,t

		;Oscillator:
		fldpi				;pi,a,t
		fmul	a_f			;f*=pi,a,t
		fmul	st1			;f*=a,a,t
		faddp	st2			;a,t+=f
		fld	st1			;t,a,t
		fsin				;sin(t),a,t

		;Output:
		fmul	st1			;a*sin(t),a,t
		fadd	dword[edi]		;y,a,t
		fstp	dword[edi]		;a,t
		add	edi, 4

		;Loop:
		fld	dword[g_k0p001]		;0.001,a,t
		fcomip	st1			;a,t
		jc	.repeat
		
		add	ebp, 12
		fstp	st0			;t
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Generates a "high-hat" sound
;-----------------------------------------------------------------------------
%define		a_d	dword[ebp+4]		;Decay factor

_synth_hihat:
		push	edi
		fldz				;0
		fld1				;a,0

		;Envelope:
.repeat:	fmul	a_d			;a*=d,0

		;Random:
		random				;r,a,0

		;Output:
		fmul	st1			;r*a,a,0
		fadd	dword[edi]		;y,a,0
		fstp	dword[edi]		;a,0
		add	edi, 4

		;Loop:
		fld	dword[g_k0p001]		;0.001,a,0
		fcomip	st1			;a,0
		jc	.repeat
		
		add	ebp, 8
		fstp	st0			;0
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Generates an sine wave
;-----------------------------------------------------------------------------
%define		a_l	dword[ebp+4]		;Length
%define		a_f	dword[ebp+8]		;Frequency

_synth_siner:
		push	edi
		fldz				;t
		mov	ecx, a_l

		;Envelope:
.repeat:	mov	dword[g_temp], ecx
		fldpi				;pi,t
		fimul	dword[g_temp]		;i*=pi,t
		fidiv	a_l			;i/=l,t
		fsin				;a,t

		;Oscillator:
		fldpi				;pi,a,t
		fmul	a_f			;f*=pi,a,t
		faddp	st2			;a,t+=f
		fld	st1			;t,a,t
		fsin				;sin(t),a,t

		;Output & loop:
		fmulp	st1			;y*=a,t
		fadd	dword[edi]		;y,t
		fstp	dword[edi]		;t
		add	edi, 4
		loop	.repeat

		add	ebp, 12
		fstp	st0			;t
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Generates an "organ" sound
;-----------------------------------------------------------------------------
%define		a_l	dword[ebp+4]		;Length
%define		a_f	dword[ebp+8]		;Frequency
%define		a_vd	dword[ebp+12]		;Vibrato depth
%define		a_vf	dword[ebp+16]		;Vibrato frequency

_synth_organ:
		push	edi
		fldz				;t
		mov	ecx, a_l

		;Vibrato:
.repeat:	mov	dword[g_temp], ecx
		fldpi				;pi,t
		fimul	dword[g_temp]		;i*=pi,t
		fmul	a_vf			;i*=vf,t
		fsin				;v,t
		fmul	a_vd			;v*=vd,t
		fmul	a_f			;v*=f,t
		fadd	a_f			;f+v,t

		;Oscillator:
		faddp	st1			;t+=f
		fld1				;1,t
		fcomip	st1			;t
		jnc	.skip			;Jump if t <= 1
		fsub	dword[g_k2]		;t-=2

		;Triangle:
.skip:		fld	st0			;t,t
		fabs				;|t|,t
		fadd	st0			;2*|t|,t
		fld1				;1,2*|t|,t
		fsubp	st1			;2*|t|-1,t

		;Envelope:
		fldpi				;pi,y,t
		fimul	dword[g_temp]		;i*=pi,y,t
		fidiv	a_l			;i/=l,y,t
		fsin				;a,y,t

		;Output & loop:
		fmulp	st1			;y*=a,t
		fadd	dword[edi]		;y,t
		fstp	dword[edi]		;t
		add	edi, 4
		loop	.repeat

		add	ebp, 20
		fstp	st0			;
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Mixes in a second buffer, starting at the write cursor
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4]		;Buffer address
%define		a_size	dword[ebp+8]		;Buffer size
%define		a_a	dword[ebp+12]		;Gain

_synth_mix:
		push	edi
		mov	ecx, a_size
		mov	esi, a_name

.repeat:	fld	dword[esi]		;y
		fmul	a_a			;y*=a
		fadd	dword[edi]		;y+=y
		fstp	dword[edi]		;
		add	edi, 4
		add	esi, 4
		loop	.repeat

		add	ebp, 16
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Creates an echo in an audio buffer
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4]		;Buffer address
%define		a_size	dword[ebp+8]		;Buffer size
%define		a_d	dword[ebp+12]		;Delay
%define		a_a	dword[ebp+16]		;Gain

_synth_echo:
		push	edi
		mov	ecx, a_size
		mov	esi, a_name
		mov	edi, esi
		sub	esi, a_d

.repeat:	fld	dword[esi]		;y
		fmul	a_a			;y*=a
		fadd	dword[edi]		;y+=y
		fstp	dword[edi]		;
		add	edi, 4
		add	esi, 4
		loop	.repeat

		add	ebp, 20
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Reduces a buffer's effective bit depth
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4] 		;Buffer address
%define		a_size	dword[ebp+8] 		;Buffer size
%define		a_d	dword[ebp+12]		;Sample levels

_synth_bitcrush:
		push	edi
		mov	ecx, a_size
		mov	esi, a_name

.repeat:	fld	dword[esi]		;x
		fmul	a_d			;x*=d
		frndint				;(int)x
		fdiv	a_d			;x/=f
		fstp	dword[esi]		;
		add	esi, 4
		loop	.repeat

		add	ebp, 16
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Bandpass filters an audio buffer
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4] 		;Buffer address
%define		a_size	dword[ebp+8] 		;Buffer size
%define		a_k	dword[ebp+12]		;Filter gain
%define		a_a1	dword[ebp+16]		;Filter coefficient
%define		a_a2	dword[ebp+20]		;Filter coefficient

_synth_bandpass:
		push	edi
		mov	ecx, a_size
		mov	esi, a_name

.repeat:	fld	dword[esi]		;x
		fmul	a_k			;x*=k
		fld	dword[esi-4]		;y1,x
		fmul	a_a1			;y1*=a1,x
		fld	dword[esi-8]		;y2,y1,x
		fmul	a_a2			;y2*=a2,y1,x
		fsubp	st1			;y1-y2,x
		faddp	st1			;x+y1-y2
		fstp	dword[esi]		;
		add	esi, 4
		loop	.repeat

		add	ebp, 24
		pop	edi
		jmp	[ebp]

;-----------------------------------------------------------------------------
; Chorus effect
;-----------------------------------------------------------------------------
%define		a_name	dword[ebp+4] 		;Buffer address
%define		a_size	dword[ebp+8] 		;Buffer size
%define		a_vd	dword[ebp+12]		;LFO depth
%define		a_vf	dword[ebp+16]		;LFO frequency
%define		a_d	dword[ebp+20]		;Delay
%define		a_a	dword[ebp+24]		;Gain

_synth_chorus:
		push	edi
		mov	ecx, a_size
		mov	esi, a_name

		;LFO:
.repeat:	mov	dword[g_temp], ecx
		fldpi				;pi,x
		fimul	dword[g_temp]		;i*=pi,x
		fmul	a_vf			;i*=vf,x
		fsin				;v,x
		fmul	a_vd			;v*=vd,x
		fadd	a_d			;d+=v,x
		fistp	dword[g_temp]		;x

		;Write delay line:
		fld	dword[esi]		;x
		mov	eax, ecx
		and	eax, s_synth_delay_size-1
		fst	dword[s_synth_delay + 4*eax]	;x

		;Read delay line:
		add	eax, [g_temp]
		and	eax, s_synth_delay_size-1
		fld	dword[s_synth_delay + 4*eax]	;y,x
		fmul	a_a			;y*=a,x
		faddp	st1			;x+=y

		;Output & loop:
		fstp	dword[esi]		;
		add	esi, 4
		loop	.repeat

		add	ebp, 28
		pop	edi
		jmp	[ebp]
