/*
 * Coded by
 *  William Swanson
 */

#include <Windows.h>
#include <gdiplus.h>
#include <math.h>
using namespace Gdiplus;

void draw(HWND aWindow);

int const k_size_x    = 256;
int const k_size_y    = 256;

float const pi = 3.1415926535897932384f;

//Global variables:
GdiplusStartupInput s_main_gdip;

/**
 * Program entry point.
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  HWND hwnd;
  ULONG gdip;
  MSG msg;

  //Create the window:
  hwnd = CreateWindowEx(
    0, "static", 0,
    WS_VISIBLE | WS_POPUP,
    (GetSystemMetrics(SM_CXSCREEN)-k_size_x)/2,
    (GetSystemMetrics(SM_CYSCREEN)-k_size_y)/2,
    k_size_x, k_size_y, 0, 0, 0, 0
  );

  //Initialize GDI+:
  GdiplusStartup(&gdip, &s_main_gdip, 0);

  //Wait for the user to press a key:
  do {
    GetMessage(&msg, NULL, 0, 0);
    if (msg.message == WM_PAINT)
      draw(msg.hwnd);
    else
      DispatchMessage(&msg);
  } while(msg.message != WM_KEYDOWN);

  //Clean up and exit:
  GdiplusShutdown(gdip);
  DestroyWindow(hwnd);
  return 0;
}

/**
 * Random number generator
 */
float random()
{
  static unsigned seed = 0x3e800000;
  seed = seed*1664525u + 1013904223u;
  return ((signed)seed)*(1.0f/0x7fffffff);
}

/**
 * Path data:
 */
unsigned char const logo_points_x[] = {0, 0, 28, 78, 95, 124, 143, 143, 171, 175, 175, 197, 201, 245, 255, 255, 255, 234, 167, 18, 0, 0, 144, 48, 44, 76, 42, 42, 144, 227, 227, 211, 211, 227, 201, 201, 166, 166, 201, 189, 182, 177, 177, 177, 184, 189, 197, 198, 198, 198, 194, 189, 160, 151, 145, 146, 147, 155, 161, 169, 173, 173, 173, 167, 160, 16, 57, 49, 46, 46, 8, 7, 10, 16, 216, 215, 214, 213, 212, 212, 213, 214, 215, 216, 217, 217, 216, 88, 152, 152, 88, 88, 234, 234, 245, 245, 234};
unsigned char const logo_points_y[] = {84, 116, 132, 157, 164, 156, 147, 118, 111, 132, 112, 102, 122, 103, 98, 69, 32, 29, 20, 0, 52, 84, 20, 11, 129, 154, 137, 8, 20, 46, 101, 107, 46, 46, 52, 79, 85, 53, 52, 113, 115, 123, 131, 140, 142, 139, 134, 126, 121, 116, 112, 113, 122, 125, 134, 144, 154, 155, 153, 150, 142, 134, 127, 120, 122, 53, 55, 69, 88, 101, 87, 73, 62, 53, 75, 76, 76, 75, 74, 72, 71, 70, 70, 71, 72, 74, 75, 55, 53, 88, 100, 55, 51, 73, 71, 50, 51};
unsigned char const logo_types[] = {0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 3, 3, 3, 3, 3, 3, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 1, 3, 3, 3, 1, 3, 3, 3, 0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1};
int const logo_count = 97;
Point logo_buffer[logo_count];

/**
 * Blur main loop
 */
void kernel(unsigned char *p)
{
  for (int i = 0; i < 4*(256*256 - 8); ++i) {
    int n =
      p[i+0] + p[i+4] + p[i+8] + p[i+12] +
      p[i+16] + p[i+20] + p[i+24] + p[i+28];
    p[i] = n/8;
  }
}

/**
 * Blurs a bitmap. The bitmap must be 256x256 in size.
 */
void blur(Bitmap *aBitmap)
{
  BitmapData data;
  aBitmap->LockBits(0, ImageLockModeRead | ImageLockModeWrite, PixelFormat32bppPARGB, &data);

  kernel(static_cast<unsigned char *>(data.Scan0));
  kernel(static_cast<unsigned char *>(data.Scan0));
  kernel(static_cast<unsigned char *>(data.Scan0));

  aBitmap->UnlockBits(&data);
  aBitmap->RotateFlip(Rotate270FlipNone);
  aBitmap->LockBits(0, ImageLockModeRead | ImageLockModeWrite, PixelFormat32bppPARGB, &data);

  kernel(static_cast<unsigned char *>(data.Scan0));
  kernel(static_cast<unsigned char *>(data.Scan0));
  kernel(static_cast<unsigned char *>(data.Scan0));

  aBitmap->UnlockBits(&data);
  aBitmap->RotateFlip(Rotate90FlipNone);
}

/**
 * Creates a logo bitmap.
 */
Image *make_logo()
{
  for (int i = 0; i < logo_count; ++i) {
    logo_buffer[i].X = static_cast<int>(logo_points_x[i]);
    logo_buffer[i].Y = static_cast<int>(logo_points_y[i]);
  }
  GraphicsPath logo(logo_buffer, logo_types, logo_count, FillModeAlternate);

  Bitmap bitmap(256, 256, PixelFormat32bppARGB);
  Graphics g(&bitmap);
  g.SetSmoothingMode(SmoothingModeHighQuality);
  g.ScaleTransform(0.75f, 0.75f, MatrixOrderPrepend);
  g.TranslateTransform(42, 42, MatrixOrderAppend);
  g.Clear(0x00000000);

  SolidBrush brush0(0xff000000);
  g.FillPath(&brush0, &logo);

  blur(&bitmap);

  g.TranslateTransform(-10, -10, MatrixOrderAppend);

  SolidBrush brush1(0xffffffff);
  g.FillPath(&brush1, &logo);

//  Pen pen(0xff000000, 1.0f);
//  pen.SetEndCap(LineCapRound);
//  pen.SetLineJoin(LineJoinRound);
//  g.DrawPath(&pen, &logo);

  g.ResetTransform();
  g.ScaleTransform(2, 2, MatrixOrderPrepend);
  g.TranslateTransform(42, 42, MatrixOrderAppend);
  g.DrawImage(static_cast<Image*>(&bitmap)->Clone(), 0, 0);

  GUID guid = {0x557CF406, 0x1A04, 0x11D3, {0x9A, 0x73, 0x00, 0x00, 0xF8, 0x1E, 0xF3, 0x2E}};
  bitmap.Save(L"texture.png", &guid);

  return static_cast<Image*>(&bitmap)->Clone();
}

/**
 * Draws to the window.
 */
void draw(HWND aWindow)
{
  PAINTSTRUCT ps;
  BeginPaint(aWindow, &ps);
  Graphics g(ps.hdc);

  g.Clear(0xff808080);
  HatchBrush brush(HatchStyleHorizontal, 0xff000000, 0xff808080);
  g.FillRectangle(&brush, 0, 0, 256, 256);

  Image *logo = make_logo();
  g.DrawImage(logo, 0, 0);
  delete logo;

  EndPaint(aWindow, &ps);
}
